@@ -10,7 +10,7 @@ use crate::{
     util::PartialBuffer,
 };
 use futures_core::ready;
-use futures_io::AsyncWrite;
+use futures_io::{AsyncBufRead, AsyncRead, AsyncWrite, IoSliceMut};
 use pin_project_lite::pin_project;
 
 #[derive(Debug)]
@@ -22,7 +22,7 @@ enum State {
 
 pin_project! {
     #[derive(Debug)]
-    pub struct Decoder<W, D: Decode> {
+    pub struct Decoder<W, D> {
         #[pin]
         writer: BufWriter<W>,
         decoder: D,
@@ -38,7 +38,9 @@ impl<W: AsyncWrite, D: Decode> Decoder<W, D> {
             state: State::Decoding,
         }
     }
+}
 
+impl<W, D> Decoder<W, D> {
     pub fn get_ref(&self) -> &W {
         self.writer.get_ref()
     }
@@ -54,7 +56,9 @@ impl<W: AsyncWrite, D: Decode> Decoder<W, D> {
     pub fn into_inner(self) -> W {
         self.writer.into_inner()
     }
+}
 
+impl<W: AsyncWrite, D: Decode> Decoder<W, D> {
     fn do_poll_write(
         self: Pin<&mut Self>,
         cx: &mut Context<'_>,
@@ -84,10 +88,7 @@ impl<W: AsyncWrite, D: Decode> Decoder<W, D> {
                 }
 
                 State::Done => {
-                    return Poll::Ready(Err(io::Error::new(
-                        io::ErrorKind::Other,
-                        "Write after end of stream",
-                    )))
+                    return Poll::Ready(Err(io::Error::other("Write after end of stream")))
                 }
             };
 
@@ -175,10 +176,37 @@ impl<W: AsyncWrite, D: Decode> AsyncWrite for Decoder<W, D> {
             ready!(self.as_mut().project().writer.as_mut().poll_close(cx))?;
             Poll::Ready(Ok(()))
         } else {
-            Poll::Ready(Err(io::Error::new(
-                io::ErrorKind::Other,
+            Poll::Ready(Err(io::Error::other(
                 "Attempt to close before finishing input",
             )))
         }
     }
 }
+
+impl<W: AsyncRead, D> AsyncRead for Decoder<W, D> {
+    fn poll_read(
+        self: Pin<&mut Self>,
+        cx: &mut Context<'_>,
+        buf: &mut [u8],
+    ) -> Poll<io::Result<usize>> {
+        self.get_pin_mut().poll_read(cx, buf)
+    }
+
+    fn poll_read_vectored(
+        self: Pin<&mut Self>,
+        cx: &mut Context<'_>,
+        bufs: &mut [IoSliceMut<'_>],
+    ) -> Poll<io::Result<usize>> {
+        self.get_pin_mut().poll_read_vectored(cx, bufs)
+    }
+}
+
+impl<W: AsyncBufRead, D> AsyncBufRead for Decoder<W, D> {
+    fn poll_fill_buf(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<io::Result<&[u8]>> {
+        self.get_pin_mut().poll_fill_buf(cx)
+    }
+
+    fn consume(self: Pin<&mut Self>, amt: usize) {
+        self.get_pin_mut().consume(amt)
+    }
+}
