@@ -32,7 +32,7 @@
 #![allow(unexpected_cfgs)]
 #![cfg_attr(
     feature = "futures-io",
-    doc = "[`futures-io`](crate::futures) | [`futures::io::AsyncBufRead`](futures_io::AsyncBufRead), [`futures::io::AsyncWrite`](futures_io::AsyncWrite)"
+    doc = "[`futures-io`] | [`futures::io::AsyncBufRead`](futures_io::AsyncBufRead), [`futures::io::AsyncWrite`](futures_io::AsyncWrite)"
 )]
 #![cfg_attr(
     not(feature = "futures-io"),
@@ -40,7 +40,7 @@
 )]
 #![cfg_attr(
     feature = "tokio",
-    doc = "[`tokio`](crate::tokio) | [`tokio::io::AsyncBufRead`](::tokio::io::AsyncBufRead), [`tokio::io::AsyncWrite`](::tokio::io::AsyncWrite)"
+    doc = "[`tokio`] | [`tokio::io::AsyncBufRead`](::tokio::io::AsyncBufRead), [`tokio::io::AsyncWrite`](::tokio::io::AsyncWrite)"
 )]
 #![cfg_attr(
     not(feature = "tokio"),
@@ -88,6 +88,14 @@
     not(feature = "gzip"),
     doc = "`gzip` (*inactive*) | `GzipEncoder`, `GzipDecoder`"
 )]
+#![cfg_attr(
+    feature = "lz4",
+    doc = "`lz4` | [`Lz4Encoder`](?search=Lz4Encoder), [`Lz4Decoder`](?search=Lz4Decoder)"
+)]
+#![cfg_attr(
+    not(feature = "lz4"),
+    doc = "`lz4` (*inactive*) | `Lz4Encoder`, `Lz4Decoder`"
+)]
 #![cfg_attr(
     feature = "lzma",
     doc = "`lzma` | [`LzmaEncoder`](?search=LzmaEncoder), [`LzmaDecoder`](?search=LzmaDecoder)"
@@ -130,6 +138,11 @@
 )]
 //!
 
+//! ## Multi-thread support
+//! The `xz` compression algorithm supports multi-threaded compression and decompression.
+//! Enable the `xz-parallel` feature to enable multi-threading support.
+//!
+
 #![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
 #![warn(
     missing_docs,
@@ -139,7 +152,12 @@
 )]
 #![cfg_attr(not(all), allow(unused))]
 
-#[cfg(any(feature = "bzip2", feature = "flate2", feature = "xz2"))]
+#[cfg(any(
+    feature = "bzip2",
+    feature = "flate2",
+    feature = "lzma",
+    feature = "lz4"
+))]
 use std::convert::TryInto;
 
 #[macro_use]
@@ -156,6 +174,8 @@ mod util;
 
 #[cfg(feature = "brotli")]
 pub mod brotli;
+#[cfg(feature = "lz4")]
+pub mod lz4;
 #[cfg(feature = "zstd")]
 pub mod zstd;
 
@@ -216,6 +236,7 @@ impl Level {
     fn into_flate2(self) -> flate2::Compression {
         let fastest = flate2::Compression::fast();
         let best = flate2::Compression::best();
+        let none = flate2::Compression::none();
 
         match self {
             Self::Fastest => fastest,
@@ -224,7 +245,7 @@ impl Level {
                 quality
                     .try_into()
                     .unwrap_or(0)
-                    .clamp(fastest.level(), best.level()),
+                    .clamp(none.level(), best.level()),
             ),
             Self::Default => flate2::Compression::default(),
         }
@@ -233,15 +254,22 @@ impl Level {
     #[cfg(feature = "zstd")]
     fn into_zstd(self) -> i32 {
         let (fastest, best) = libzstd::compression_level_range().into_inner();
+
+        // NOTE: zstd's "fastest" level is -131072 which can create outputs larger than inputs.
+        // This library chooses a "fastest" level which has a more-or-less equivalent compression
+        // ratio to gzip's fastest mode. We still allow precise levels to go negative.
+        // See discussion in https://github.com/Nullus157/async-compression/issues/352
+        const OUR_FASTEST: i32 = 1;
+
         match self {
-            Self::Fastest => fastest,
+            Self::Fastest => OUR_FASTEST,
             Self::Best => best,
             Self::Precise(quality) => quality.clamp(fastest, best),
             Self::Default => libzstd::DEFAULT_COMPRESSION_LEVEL,
         }
     }
 
-    #[cfg(feature = "xz2")]
+    #[cfg(feature = "lzma")]
     fn into_xz2(self) -> u32 {
         match self {
             Self::Fastest => 0,
@@ -250,4 +278,20 @@ impl Level {
             Self::Default => 5,
         }
     }
+
+    #[cfg(feature = "lz4")]
+    fn into_lz4(
+        self,
+        mut preferences: ::lz4::liblz4::LZ4FPreferences,
+    ) -> ::lz4::liblz4::LZ4FPreferences {
+        let level = match self {
+            Self::Fastest => 0,
+            Self::Best => 12,
+            Self::Precise(quality) => quality.try_into().unwrap_or(0).min(12),
+            Self::Default => 0,
+        };
+
+        preferences.compression_level = level;
+        preferences
+    }
 }
