@@ -11,18 +11,19 @@ use crate::{
 };
 use futures_core::ready;
 use pin_project_lite::pin_project;
-use tokio::io::AsyncWrite;
+use tokio::io::{AsyncBufRead, AsyncRead, AsyncWrite, ReadBuf};
 
 #[derive(Debug)]
 enum State {
     Encoding,
+    Flushing,
     Finishing,
     Done,
 }
 
 pin_project! {
     #[derive(Debug)]
-    pub struct Encoder<W, E: Encode> {
+    pub struct Encoder<W, E> {
         #[pin]
         writer: BufWriter<W>,
         encoder: E,
@@ -39,6 +40,16 @@ impl<W: AsyncWrite, E: Encode> Encoder<W, E> {
         }
     }
 
+    pub fn with_capacity(writer: W, encoder: E, cap: usize) -> Self {
+        Self {
+            writer: BufWriter::with_capacity(cap, writer),
+            encoder,
+            state: State::Encoding,
+        }
+    }
+}
+
+impl<W, E> Encoder<W, E> {
     pub fn get_ref(&self) -> &W {
         self.writer.get_ref()
     }
@@ -58,7 +69,9 @@ impl<W: AsyncWrite, E: Encode> Encoder<W, E> {
     pub fn into_inner(self) -> W {
         self.writer.into_inner()
     }
+}
 
+impl<W: AsyncWrite, E: Encode> Encoder<W, E> {
     fn do_poll_write(
         self: Pin<&mut Self>,
         cx: &mut Context<'_>,
@@ -76,11 +89,14 @@ impl<W: AsyncWrite, E: Encode> Encoder<W, E> {
                     State::Encoding
                 }
 
+                // Once a flush has been started, it must be completed.
+                State::Flushing => match this.encoder.flush(&mut output)? {
+                    true => State::Encoding,
+                    false => State::Flushing,
+                },
+
                 State::Finishing | State::Done => {
-                    return Poll::Ready(Err(io::Error::new(
-                        io::ErrorKind::Other,
-                        "Write after shutdown",
-                    )))
+                    return Poll::Ready(Err(io::Error::other("Write after shutdown")))
                 }
             };
 
@@ -101,20 +117,19 @@ impl<W: AsyncWrite, E: Encode> Encoder<W, E> {
             let mut output = PartialBuffer::new(output);
 
             let done = match this.state {
-                State::Encoding => this.encoder.flush(&mut output)?,
+                State::Encoding | State::Flushing => this.encoder.flush(&mut output)?,
 
                 State::Finishing | State::Done => {
-                    return Poll::Ready(Err(io::Error::new(
-                        io::ErrorKind::Other,
-                        "Flush after shutdown",
-                    )))
+                    return Poll::Ready(Err(io::Error::other("Flush after shutdown")))
                 }
             };
+            *this.state = State::Flushing;
 
             let produced = output.written().len();
             this.writer.as_mut().produce(produced);
 
             if done {
+                *this.state = State::Encoding;
                 return Poll::Ready(Ok(()));
             }
         }
@@ -136,6 +151,12 @@ impl<W: AsyncWrite, E: Encode> Encoder<W, E> {
                     }
                 }
 
+                // Once a flush has been started, it must be completed.
+                State::Flushing => match this.encoder.flush(&mut output)? {
+                    true => State::Finishing,
+                    false => State::Flushing,
+                },
+
                 State::Done => State::Done,
             };
 
@@ -179,3 +200,23 @@ impl<W: AsyncWrite, E: Encode> AsyncWrite for Encoder<W, E> {
         Poll::Ready(Ok(()))
     }
 }
+
+impl<W: AsyncRead, E> AsyncRead for Encoder<W, E> {
+    fn poll_read(
+        self: Pin<&mut Self>,
+        cx: &mut Context<'_>,
+        buf: &mut ReadBuf<'_>,
+    ) -> Poll<io::Result<()>> {
+        self.get_pin_mut().poll_read(cx, buf)
+    }
+}
+
+impl<W: AsyncBufRead, E> AsyncBufRead for Encoder<W, E> {
+    fn poll_fill_buf(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<io::Result<&[u8]>> {
+        self.get_pin_mut().poll_fill_buf(cx)
+    }
+
+    fn consume(self: Pin<&mut Self>, amt: usize) {
+        self.get_pin_mut().consume(amt)
+    }
+}
